import { EventsHandler, IEventHandler } from '@nestjs/cqrs';
import { TransactionChangeStatusEvent } from '../../entities/event/transaction-change-status.event';
import { SalesPriceFormulaDataService } from 'src/modules/transaction/sales-price-formula/data/services/sales-price-formula-data.service';
import { TaxDataService } from 'src/modules/transaction/tax/data/services/tax-data.service';
import { FormulaType } from 'src/modules/transaction/sales-price-formula/constants';
import * as math from 'mathjs';
import { Equation, parse } from 'algebra.js';
import { STATUS } from 'src/core/strings/constants/base.constants';
import { TransactionDataService } from '../../../data/services/transaction-data.service';
import { TransactionModel } from '../../../data/models/transaction.model';

@EventsHandler(TransactionChangeStatusEvent)
export class SettledTransactionHandler
  implements IEventHandler<TransactionChangeStatusEvent>
{
  constructor(
    private formulaService: SalesPriceFormulaDataService,
    private taxService: TaxDataService,
    private dataService: TransactionDataService,
  ) {}

  async handle(event: TransactionChangeStatusEvent) {
    const old_data = event.data.old;
    const data = event.data.data;

    if (
      old_data.status == data.status ||
      ![STATUS.ACTIVE, STATUS.SETTLED].includes(data.status)
    )
      return;

    const profit_formula = await this.formulaService.getOneByOptions({
      where: {
        type: FormulaType.PROFIT_SHARE,
      },
    });

    const sales_price = await this.formulaService.getOneByOptions({
      where: {
        type: FormulaType.SALES_PRICE,
      },
    });

    const taxes = await this.taxService.getManyByOptions({
      where: {
        status: STATUS.ACTIVE,
      },
    });

    const queryRunner = this.dataService
      .getRepository()
      .manager.connection.createQueryRunner();

    // const profit_share_value = this.calculateFormula(profit_formula.formula_string, taxes, data.payment_total_net_profit ?? 0);
    // const sale_price_value = this.calculateFormula(sales_price.formula_string, taxes, data.payment_total_net_profit ?? 0);

    Object.assign(data, {
      profit_share_formula: profit_formula.formula_string,
      sales_price_formula: sales_price.formula_string,
    });

    await this.dataService.create(queryRunner, TransactionModel, data);
  }

  calculateFormula(formula, taxes, total) {
    const regex = /([a-zA-Z0-9_]+)/g;
    const variable = {};

    const matches: string[] = formula.match(regex);
    const uniqueMatches = new Set(matches);
    const keys = Array.from(uniqueMatches).filter((key) => key != 'dpp');

    for (const key of keys) {
      const keyData = taxes.find((tax) => tax.name == key);
      variable[key] = keyData.value / 100;
    }

    try {
      const x1 = math.simplify(formula, variable).toString();
      console.log('Formula ', x1);
      const dppFormula = parse(x1);
      const totalFormula = parse(total.toString());
      const equation = new Equation(totalFormula, dppFormula);

      console.log(equation.toString());
      const result = equation.solveFor('dpp').toString();
      console.log(result);

      const value = math.evaluate(result);
      console.log(value);
      return value;
    } catch (e) {
      console.log(e);
    }
  }
}
